//
// Fonts.cpp : ChooseFont
//

#include "stdafx.h"
#include "TinyPad.h"
#include <commctrl.h>
#include <aygshell.h>
#include <sipapi.h>

#include <commdlg.h>

#include "fonts.h"

extern HINSTANCE		g_hInst;
extern HWND				hWnd;
extern TCHAR			szTitle[];

static const int		FontSizes[] = {
	3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 14, 16, 18, 20, 24, 28, 32, 48, 72,
};

#define nFontSizes	(sizeof(FontSizes) / sizeof(FontSizes[0]))

static LPCHOOSEFONT		m_lpcf;
static LPLOGFONT		m_lplf;

static LPLOGFONT		m_pFonts;
static int				m_nFonts;

static int				m_fontindex;

/*
 *	_atoi
 */
int _atoi(TCHAR **pp)
{
	TCHAR	*p = *pp;
	int		n = 0;
	BOOL	minus = FALSE;

	if (*p == L'-') {
		minus = TRUE;
		p++;
	}

	while (*p >= L'0' && *p <= L'9')
		n = (n * 10) + (*p++ - L'0');

	*pp = p;

	return minus ? -n : n;
}

/*
 *	FontProc
 */
static int CALLBACK FontProc(LPLOGFONT plf, LPTEXTMETRIC ptm, DWORD dwType, LPARAM lpData)
{
	HWND		hfontname;

	hfontname = GetDlgItem((HWND)lpData, IDC_FONTNAME);
	if (SendMessage(hfontname, CB_ADDSTRING, 0, (LPARAM)(plf->lfFaceName)) < 0) {
		MessageBox((HWND)lpData, TEXT("CB_ADDSTRING: failed."), szTitle, MB_OK);
		return 0;
	}

	// Allocate buffer and copy logfont
	if (m_pFonts)
		m_pFonts = (LPLOGFONT)LocalReAlloc(m_pFonts, sizeof(*m_pFonts) * (m_nFonts + 1), LMEM_MOVEABLE);
	else
		m_pFonts = (LPLOGFONT)LocalAlloc(LMEM_FIXED, sizeof(*m_pFonts));

	if (!m_pFonts)
		return 0;
	m_pFonts[m_nFonts] = *plf;

	m_nFonts++;

	return 1;
}

/*
 *	SelectFont dialog
 */
static LRESULT CALLBACK SelectFont(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	SHINITDLGINFO shidi;
	HDC hdc;
	int i;
	TCHAR buf[256];
	HWND hfontname;
	HWND hfontsize;
	int point;
	int state;
	int index;
	TCHAR *pbuf;

	switch (message)
	{
		case WM_INITDIALOG:
			// Create a Done button and size it.  
			shidi.dwMask = SHIDIM_FLAGS;
			shidi.dwFlags = SHIDIF_DONEBUTTON | SHIDIF_SIPDOWN | SHIDIF_SIZEDLGFULLSCREEN;
			shidi.hDlg = hDlg;
			SHInitDialog(&shidi);

			// Enum fonts
			m_pFonts = NULL;
			m_nFonts = 0;

			hdc = GetDC(hDlg);
			EnumFonts(hdc, NULL, (FONTENUMPROC)FontProc, (LPARAM)hDlg);

			hfontname = GetDlgItem(hDlg, IDC_FONTNAME);
			for (i = 0; i < m_nFonts; i++) {
				SendMessage(hfontname, CB_GETLBTEXT, i, (LPARAM)buf);
				if (wcscmp(buf, m_lplf->lfFaceName) == 0) {
					SendMessage(hfontname, CB_SETCURSEL, i, 0);
					break;
				}
			}

			// Font sizes
			point = MulDiv((m_lplf->lfHeight < 0 ? -m_lplf->lfHeight : m_lplf->lfHeight),
				72, GetDeviceCaps(hdc, LOGPIXELSY));
			hfontsize = GetDlgItem(hDlg, IDC_FONTSIZE);
			for (i = 0; i < nFontSizes; i++) {
				wsprintf(buf, TEXT("%d"), FontSizes[i]);
				SendMessage(hfontsize, CB_ADDSTRING, 0, (LPARAM)buf);
				if (FontSizes[i] == point)
					SendMessage(hfontsize, CB_SETCURSEL, i, 0);
			}

			// Styles
			state = (m_lplf->lfWeight > FW_NORMAL) ? BST_CHECKED : BST_UNCHECKED;
			SendDlgItemMessage(hDlg, IDC_BOLD, BM_SETCHECK, state, 0);

			state = (m_lplf->lfItalic) ? BST_CHECKED : BST_UNCHECKED;
			SendDlgItemMessage(hDlg, IDC_ITALIC, BM_SETCHECK, state, 0);

			return TRUE; 

		case WM_COMMAND:
			if (LOWORD(wParam) == IDOK)
			{
				PLOGFONT plf;
				BOOL dirty;

				// Update logfont
				hdc = GetDC(hDlg);
				dirty = FALSE;

				hfontname = GetDlgItem(hDlg, IDC_FONTNAME);
				index = SendMessage(hfontname, CB_GETCURSEL, 0, 0);
				SendMessage(hfontname, CB_GETLBTEXT, index, (LPARAM)buf);

				// Find target logfont
				for (i = 0; i < m_nFonts; i++) {
					if (wcscmp(buf, m_pFonts[i].lfFaceName) == 0)
						break;
				}
				if (i >= m_nFonts)
					break;
				plf = &m_pFonts[i];

				// Dont care the width
				plf->lfWidth = 0;

				// Check new name or not
				if (wcscmp(buf, m_lplf->lfFaceName) != 0)
					dirty = TRUE;

				// Retrieve size and styles
				hfontsize = GetDlgItem(hDlg, IDC_FONTSIZE);
				index = SendMessage(hfontsize, CB_GETCURSEL, 0, 0);
				SendMessage(hfontsize, CB_GETLBTEXT, index, (LPARAM)buf);
				pbuf = buf;
				point = _atoi(&pbuf);
				plf->lfHeight = -MulDiv(point, GetDeviceCaps(hdc, LOGPIXELSY), 72);
				if (plf->lfHeight != m_lplf->lfHeight)
					dirty = TRUE;

				state = SendDlgItemMessage(hDlg, IDC_BOLD, BM_GETCHECK, 0, 0);
				plf->lfWeight = (state == BST_CHECKED) ? FW_BOLD : FW_NORMAL;
				if (plf->lfWeight  != m_lplf->lfWeight)
					dirty = TRUE;

				state = SendDlgItemMessage(hDlg, IDC_ITALIC, BM_GETCHECK, 0, 0);
				plf->lfItalic = (state == BST_CHECKED) ? TRUE : FALSE;
				if (plf->lfItalic != m_lplf->lfItalic)
					dirty = TRUE;

				// Update new logfont if changed
				if (dirty)
					*m_lplf = *plf;

				// Free logfont buffer
				LocalFree(m_pFonts);

				EndDialog(hDlg, (WPARAM)dirty);
				return TRUE;
			}
			break;
	}
    return FALSE;
}


/*
 *	ChooseFont
 */
BOOL APIENTRY ChooseFont(LPCHOOSEFONT lpcf)
{
	m_lpcf = lpcf;
	m_lplf = lpcf->lpLogFont;

	return DialogBox(g_hInst, (LPCTSTR)IDD_SELECTFONT, hWnd, (DLGPROC)SelectFont);
}

// vi: ts=4 sw=4
